@@ -1,7 +1,8 @@
-import unittest, time, traceback, sys
+import unittest
 
 import apport
 
+
 class T(unittest.TestCase):
     def test_get_uninstalled_package(self):
         '''get_uninstalled_package()'''
@@ -13,4 +14,3 @@ class T(unittest.TestCase):
         self.assertTrue(apport.packaging.is_distro_package(p))
 
 unittest.main()
-
