@@ -1,5 +1,11 @@
 # coding: UTF-8
-import unittest, shutil, time, tempfile, os, subprocess, grp, atexit, re
+import unittest, shutil, time, tempfile, os, subprocess, grp, atexit, re, sys
+
+try:
+    from cStringIO import StringIO
+    StringIO  # pyflakes
+except ImportError:
+    from io import StringIO
 
 import apport.report
 import problem_report
@@ -733,6 +739,74 @@ $0.bin 2>/dev/null
         self._validate_gdb_fields(pr)
         self.assertFalse('AssertionMessage' in pr, pr.get('AssertionMessage'))
 
+    def test_add_gdb_info_abort_glib(self):
+        '''add_gdb_info() with glib assertion'''
+        (fd, script) = tempfile.mkstemp()
+        assert not os.path.exists('core')
+        try:
+            os.close(fd)
+
+            # create a test script which produces a core dump for us
+            with open(script, 'w') as fd:
+                fd.write('''#!/bin/sh
+gcc -o $0.bin -x c - `pkg-config --cflags --libs glib-2.0` <<EOF
+#include <glib.h>
+int main() { g_assert_cmpint(1, <, 0); }
+EOF
+ulimit -c unlimited
+$0.bin 2>/dev/null
+''')
+            # call script and verify that it gives us a proper ELF core dump
+            assert subprocess.call(['/bin/sh', script]) != 0
+            self._validate_core('core')
+
+            pr = apport.report.Report()
+            pr['ExecutablePath'] = script + '.bin'
+            pr['CoreDump'] = ('core',)
+            pr.add_gdb_info()
+        finally:
+            os.unlink(script)
+            os.unlink(script + '.bin')
+            os.unlink('core')
+
+        self._validate_gdb_fields(pr)
+        self.assertTrue(pr['AssertionMessage'].startswith('ERROR:<stdin>:2:main: assertion failed (1 < 0):'),
+                        pr['AssertionMessage'])
+
+    def test_add_gdb_info_abort_libnih(self):
+        '''add_gdb_info() with libnih assertion'''
+        (fd, script) = tempfile.mkstemp()
+        assert not os.path.exists('core')
+        try:
+            os.close(fd)
+
+            # create a test script which produces a core dump for us
+            with open(script, 'w') as fd:
+                fd.write('''#!/bin/sh
+gcc -o $0.bin -x c - `pkg-config --cflags --libs libnih` <<EOF
+#include <libnih.h>
+int main() { nih_assert (1 < 0); }
+EOF
+ulimit -c unlimited
+$0.bin 2>/dev/null
+''')
+            # call script and verify that it gives us a proper ELF core dump
+            assert subprocess.call(['/bin/sh', script]) != 0
+            self._validate_core('core')
+
+            pr = apport.report.Report()
+            pr['ExecutablePath'] = script + '.bin'
+            pr['CoreDump'] = ('core',)
+            pr.add_gdb_info()
+        finally:
+            os.unlink(script)
+            os.unlink(script + '.bin')
+            os.unlink('core')
+
+        self._validate_gdb_fields(pr)
+        self.assertTrue('Assertion failed in main: 1 < 0' in pr['AssertionMessage'],
+                        pr['AssertionMessage'])
+
     def test_search_bug_patterns(self):
         '''search_bug_patterns().'''
 
@@ -1034,6 +1108,66 @@ def add_info(report, ui):
             apport.report._common_hook_dir = orig_common_hook_dir
             apport.report._opt_dir = orig_opt_dir
 
+    def test_add_hooks_info_errors(self):
+        '''add_hooks_info() with errors in hooks'''
+
+        orig_hook_dir = apport.report._hook_dir
+        apport.report._hook_dir = tempfile.mkdtemp()
+        orig_common_hook_dir = apport.report._common_hook_dir
+        apport.report._common_hook_dir = tempfile.mkdtemp()
+        orig_stderr = sys.stderr
+        sys.stderr = StringIO()
+        try:
+            with open(os.path.join(apport.report._hook_dir, 'fooprogs.py'), 'w') as fd:
+                fd.write('''def add_info(report, ui):
+    report['BinHookBefore'] = '1'
+    1/0
+    report['BinHookAfter'] = '1'
+''')
+            with open(os.path.join(apport.report._hook_dir, 'source_foo.py'), 'w') as fd:
+                fd.write('''def add_info(report, ui):
+    report['SourceHookBefore'] = '1'
+    unknown()
+    report['SourceHookAfter'] = '1'
+''')
+
+            r = apport.report.Report()
+            r['Package'] = 'fooprogs 0.2'
+            r['SourcePackage'] = 'foo'
+            r['ExecutablePath'] = '/bin/foo-cli'
+
+            self.assertEqual(r.add_hooks_info('fake_ui'), False)
+
+            # should have the data until the crash
+            self.assertEqual(r['BinHookBefore'], '1')
+            self.assertEqual(r['SourceHookBefore'], '1')
+
+            # should print the exceptions to stderr
+            err = sys.stderr.getvalue()
+            self.assertTrue('ZeroDivisionError:' in err, err)
+            self.assertTrue("global name 'unknown' is not defined" in err, err)
+
+            # should also add the exceptions to the report
+            self.assertTrue('NameError:' in r['HookError_source_foo'],
+                            r['HookError_source_foo'])
+            self.assertTrue('line 3, in add_info' in r['HookError_source_foo'],
+                            r['HookError_source_foo'])
+            self.assertFalse('ZeroDivisionError' in r['HookError_source_foo'],
+                             r['HookError_source_foo'])
+
+            self.assertTrue('ZeroDivisionError:' in r['HookError_fooprogs'],
+                            r['HookError_fooprogs'])
+            self.assertTrue('line 3, in add_info' in r['HookError_source_foo'],
+                            r['HookError_fooprogs'])
+            self.assertFalse('NameError:' in r['HookError_fooprogs'],
+                             r['HookError_fooprogs'])
+        finally:
+            sys.stderr = orig_stderr
+            shutil.rmtree(apport.report._hook_dir)
+            shutil.rmtree(apport.report._common_hook_dir)
+            apport.report._hook_dir = orig_hook_dir
+            apport.report._common_hook_dir = orig_common_hook_dir
+
     def test_ignoring(self):
         '''mark_ignore() and check_ignored().'''
 
@@ -1747,6 +1881,57 @@ RUNQUEUES[0]: c6002320
         r['AssertionMessage'] = 'foo.c:42 main: i > 0'
         self.assertEqual(r.crash_signature(), '/bin/bash:foo.c:42 main: i > 0')
 
+        # kernel oops
+        report = apport.report.Report('KernelOops')
+        report['OopsText'] = '''
+BUG: unable to handle kernel paging request at ffffb4ff
+IP: [<c11e4690>] ext4_get_acl+0x80/0x210
+*pde = 01874067 *pte = 00000000
+Oops: 0000 [#1] SMP
+Modules linked in: bnep rfcomm bluetooth dm_crypt olpc_xo1 scx200_acb snd_cs5535audio snd_ac97_codec ac97_bus snd_pcm snd_seq_midi snd_rawmidi snd_seq_midi_event snd_seq snd_timer snd_seq_device snd cs5535_gpio soundcore snd_page_alloc binfmt_misc geode_aes cs5535_mfd geode_rng msr vesafb usbhid hid 8139too pata_cs5536 8139cp
+
+Pid: 1798, comm: gnome-session-c Not tainted 3.0.0-11-generic #17-Ubuntu First International Computer, Inc.  ION603/ION603
+EIP: 0060:[<c11e4690>] EFLAGS: 00010286 CPU: 0
+EIP is at ext4_get_acl+0x80/0x210
+EAX: f5d3009c EBX: f5d30088 ECX: 00000000 EDX: f5d301d8
+ESI: ffffb4ff EDI: 00008000 EBP: f29b3dc8 ESP: f29b3da4
+ DS: 007b ES: 007b FS: 00d8 GS: 00e0 SS: 0068
+Process gnome-session-c (pid: 1798, ti=f29b2000 task=f2bd72c0 task.ti=f29b2000)
+Stack:
+ f29b3db0 c113bb90 f5d301d8 f29b3de4 c11b9016 f5d3009c f5d30088 f5d30088
+ 00000001 f29b3ddc c11e4cca 00000001 f5d30088 000081ed f29b3df0 c11313b7
+ 00000021 00000021 f5d30088 f29b3e08 c1131b45 c11e4c80 f5d30088 00000021
+Call Trace:
+ [<c113bb90>] ? d_splice_alias+0x40/0x50
+ [<c11b9016>] ? ext4_lookup.part.30+0x56/0x120
+ [<c11e4cca>] ext4_check_acl+0x4a/0x90
+ [<c11313b7>] acl_permission_check+0x97/0xa0
+ [<c1131b45>] generic_permission+0x25/0xc0
+ [<c11e4c80>] ? ext4_xattr_set_acl+0x160/0x160
+ [<c1131c79>] inode_permission+0x99/0xd0
+ [<c11e4c80>] ? ext4_xattr_set_acl+0x160/0x160
+ [<c1131d1b>] may_open+0x6b/0x110
+ [<c1134566>] do_last+0x1a6/0x640
+ [<c113595d>] path_openat+0x9d/0x350
+ [<c10de692>] ? unlock_page+0x42/0x50
+ [<c10fb960>] ? __do_fault+0x3b0/0x4b0
+ [<c1135c41>] do_filp_open+0x31/0x80
+ [<c124c743>] ? aa_dup_task_context+0x33/0x60
+ [<c1250eed>] ? apparmor_cred_prepare+0x2d/0x50
+ [<c112e9ef>] open_exec+0x2f/0x110
+ [<c112eef7>] ? check_unsafe_exec+0xb7/0xf0
+ [<c112efba>] do_execve_common+0x8a/0x270
+ [<c112f1b7>] do_execve+0x17/0x20
+ [<c100a0a7>] sys_execve+0x37/0x70
+ [<c15336ae>] ptregs_execve+0x12/0x18
+ [<c152c8d4>] ? syscall_call+0x7/0xb
+Code: 8d 76 00 8d 93 54 01 00 00 8b 32 85 f6 74 e2 8d 43 14 89 55 e4 89 45 f0 e8 2e 7e 34 00 8b 55 e4 8b 32 83 fe ff 74 07 85 f6 74 03 <3e> ff 06 8b 45 f0 e8 25 19 e4 ff 90 83 fe ff 75 b5 81 ff 00 40
+EIP: [<c11e4690>] ext4_get_acl+0x80/0x210 SS:ESP 0068:f29b3da4
+CR2: 00000000ffffb4ff
+---[ end trace b567e6a3070ffb42 ]---
+'''
+        self.assertEqual(report.crash_signature(), 'kernel paging request:ext4_get_acl+0x80/0x210:ext4_check_acl+0x4a/0x90:acl_permission_check+0x97/0xa0:generic_permission+0x25/0xc0:inode_permission+0x99/0xd0:may_open+0x6b/0x110:do_last+0x1a6/0x640:path_openat+0x9d/0x350:do_filp_open+0x31/0x80:open_exec+0x2f/0x110:do_execve_common+0x8a/0x270:do_execve+0x17/0x20:sys_execve+0x37/0x70:ptregs_execve+0x12/0x18')
+
     def test_nonascii_data(self):
         '''methods get along with non-ASCII data'''
 
@@ -1798,10 +1983,34 @@ ffffffffff600000-ffffffffff601000 r-xp 00000000 00:00 0                  [vsysca
         self.assertEqual(pr._address_to_offset(0x006de000), '/bin/bash+0')
         self.assertEqual(pr._address_to_offset(0x006df000), '/bin/bash+1000')
         self.assertEqual(pr._address_to_offset(0x006df001), None)
+        self.assertEqual(pr._address_to_offset(0), None)
+        self.assertEqual(pr._address_to_offset(0x10), None)
 
         self.assertEqual(pr._address_to_offset(0x7f491fc24010),
                          '/lib/with spaces !/libfoo.so+10')
 
+    def test_address_to_offset_arm(self):
+        '''_address_to_offset() for ARM /proc/pid/maps'''
+
+        pr = apport.report.Report()
+        pr['ProcMaps'] = '''
+00008000-0000e000 r-xp 00000000 08:01 13243326   /usr/lib/dconf/dconf-service
+00017000-00038000 rw-p 00000000 00:00 0          [heap]
+40017000-4001d000 rw-p 00000000 00:00 0
+40026000-400f2000 r-xp 00000000 08:01 13110792   /usr/lib/arm-linux-gnueabihf/libgio-2.0.so.0.3400.0
+400f2000-400f9000 ---p 000cc000 08:01 13110792   /usr/lib/arm-linux-gnueabihf/libgio-2.0.so.0.3400.0
+4020d000-4020f000 rw-p 00000000 00:00 0
+4020f000-402e5000 r-xp 00000000 08:01 13108294   /lib/arm-linux-gnueabihf/libc-2.15.so
+402e5000-402ed000 ---p 000d6000 08:01 13108294   /lib/arm-linux-gnueabihf/libc-2.15.so
+40d21000-40e00000 ---p 00000000 00:00 0
+befdf000-bf000000 rw-p 00000000 00:00 0          [stack]
+ffff0000-ffff1000 r-xp 00000000 00:00 0          [vectors]
+'''
+        self.assertEqual(pr._address_to_offset(0x402261e6),
+                         '/lib/arm-linux-gnueabihf/libc-2.15.so+171e6')
+        self.assertEqual(pr._address_to_offset(0x4002601F),
+                         '/usr/lib/arm-linux-gnueabihf/libgio-2.0.so.0.3400.0+1f')
+
     def test_address_to_offset_live(self):
         '''_address_to_offset() for current /proc/pid/maps'''
 
@@ -1870,7 +2079,27 @@ No symbol table info available.
 #6  0x000000000041d715 in main ()
 #7  0x000000000041d703 in _start ()
 '''
-        self.assertNotEqual(pr.crash_signature_addresses(), None)
+        sig = pr.crash_signature_addresses()
+        self.assertNotEqual(sig, None)
+
+        # one true unresolvable, and some "low address" artifacts; should be
+        # identical to the one above
+        pr['Stacktrace'] = '''
+#0  0x00007f491fac5687 in kill () at ../sysdeps/unix/syscall-template.S:82
+No locals.
+#1  0x000001000043fd51 in kill_pid ()
+#2  0x0000000000000010 in ??
+#3  g_main_context_iterate (context=0x1731680) at gmain.c:3068
+#4  0x000000000042eb76 in ?? ()
+#5  0x0000000000000000 in ?? ()
+#6  0x0000000000000421 in ?? ()
+#7  0x00000000004324d8 in ??
+No symbol table info available.
+#8  0x00000000004707e3 in parse_and_execute ()
+#9  0x000000000041d715 in main ()
+#10 0x000000000041d703 in _start ()
+'''
+        self.assertEqual(pr.crash_signature_addresses(), sig)
 
         # two unresolvables, 2/7 is too much
         pr['Stacktrace'] = '''
